@@ -9,7 +9,7 @@ use Exporter::Tiny ();
 package Lexical::Accessor;
 
 use Carp qw( carp croak );
-use Hash::Util::FieldHash::Compat qw( fieldhash );
+use Hash::FieldHash qw( fieldhash );
 use Scalar::Util qw( blessed reftype );
 
 BEGIN {
@@ -17,7 +17,7 @@ BEGIN {
 };
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.001';
+our $VERSION   = '0.002';
 our @EXPORT    = qw/ lexical_has /;
 our @ISA       = qw/ Exporter::Tiny /;
 
@@ -82,7 +82,7 @@ sub lexical_has : method
 		push @return, $writer if $is eq 'rwp';
 	}
 	
-	if ($opts{accessor} or $is eq 'rw' or not defined $is)
+	if ($opts{accessor} or $is eq 'rw')
 	{
 		my $accessor = $me->_lexical_accessor($name, $uniq, \%opts);
 		${ $opts{accessor} } = $accessor if ref $opts{accessor};
@@ -97,7 +97,13 @@ sub _canonicalize_opts : method
 	my $me = shift;
 	my ($name, $uniq, $opts) = @_;
 	
-	$opts->{is} ||= 'bare';
+	$opts->{is} ||= 'rw';
+	
+	if ($opts->{is} eq 'lazy')
+	{
+		$opts->{is}      = 'ro';
+		$opts->{builder} = 1 unless $opts->{builder} || $opts->{default};
+	}
 	
 	croak("Initializers are not supported") if $opts->{initializer};
 	croak("Traits are not supported") if $opts->{traits};
@@ -391,11 +397,35 @@ sub _lexical_writer : method
 sub _inline_lexical_writer : method
 {
 	my $me = shift;
-	my $get = $me->_inline_lexical_access(@_);
+	my ($name, $uniq, $opts) = @_;
+	
+	my $get    = $me->_inline_lexical_access(@_);
+	my $coerce = $me->_inline_lexical_type_coercion('$_[1]', @_);
+	
+	if ($coerce eq '$_[1]')  # i.e. no coercion
+	{
+		if (!$opts->{trigger} and !$opts->{weak_ref})
+		{
+			return sprintf(
+				'%s = %s',
+				$get,
+				$me->_inline_lexical_type_assertion('$_[1]', @_),
+			);
+		}
+		
+		return sprintf(
+			'%s; %s; %s = $_[1]; %s; %s',
+			$me->_inline_lexical_type_assertion('$_[1]', @_),
+			$me->_inline_lexical_trigger('$_[1]', $get, @_),
+			$get,
+			$me->_inline_lexical_weaken(@_),
+			$me->_inline_lexical_get(@_),
+		);
+	}
 	
 	sprintf(
 		'my $val = %s; %s; %s; %s = $val; %s; $val',
-		$me->_inline_lexical_type_coercion('$_[1]', @_),
+		$coerce,
 		$me->_inline_lexical_type_assertion('$val', @_),
 		$me->_inline_lexical_trigger('$val', $get, @_),
 		$get,
@@ -416,11 +446,36 @@ sub _lexical_accessor : method
 sub _inline_lexical_accessor : method
 {
 	my $me = shift;
-	my $get = $me->_inline_lexical_access(@_);
+	my ($name, $uniq, $opts) = @_;
+	
+	my $get    = $me->_inline_lexical_access(@_);
+	my $coerce = $me->_inline_lexical_type_coercion('$_[1]', @_);
+	
+	if ($coerce eq '$_[1]')  # i.e. no coercion
+	{
+		if (!$opts->{trigger} and !$opts->{weak_ref})
+		{
+			return sprintf(
+				'(@_ > 1) ? (%s = %s) : %s',
+				$get,
+				$me->_inline_lexical_type_assertion('$_[1]', @_),
+				$get,
+			);
+		}
+		
+		return sprintf(
+			'if (@_ > 1) { %s; %s; %s = $_[1]; %s }; %s',
+			$me->_inline_lexical_type_assertion('$_[1]', @_),
+			$me->_inline_lexical_trigger('$_[1]', $get, @_),
+			$get,
+			$me->_inline_lexical_weaken(@_),
+			$me->_inline_lexical_get(@_),
+		);
+	}
 	
 	sprintf(
 		'if (@_ > 1) { my $val = %s; %s; %s; %s = $val; %s }; %s',
-		$me->_inline_lexical_type_coercion('$_[1]', @_),
+		$coerce,
 		$me->_inline_lexical_type_assertion('$val', @_),
 		$me->_inline_lexical_trigger('$val', $get, @_),
 		$get,
@@ -468,14 +523,15 @@ sub _inline_lexical_type_coercion : method
 		return $coercion->inline_coercion($var);
 	}
 	
+	# Otherwise need to close over $coerce
+	$opts->{inline_environment}{'$coercion'} = \$coercion;
+	
 	if ( blessed($coercion)
 	and $coercion->can('coerce') )
 	{
-		$opts->{inline_environment}{'$coercion'} = \$coercion;
 		return sprintf('$coercion->coerce(%s)', $var);
 	}
 	
-	$opts->{inline_environment}{'$coercion'} = \$coercion;
 	return sprintf('$coercion->(%s)', $var);
 }
 
@@ -484,33 +540,44 @@ sub _inline_lexical_type_assertion : method
 	my $me = shift;
 	my ($var, $name, $uniq, $opts) = @_;
 	
-	my $type = $opts->{isa} or return '';
+	my $type = $opts->{isa} or return $var;
 	
 	if ( blessed($type)
-	and $type->can('can_be_inlined')
-	and $type->can_be_inlined
-	and $type->can('inline_assert') )
+	and $type->isa('Type::Tiny')
+	and $type->can_be_inlined )
 	{
-		return $type->inline_assert($var);
+		my $ass = $type->inline_assert($var);
+		if ($ass =~ /\Ado \{(.+)\};\z/sm)
+		{
+			return "do { $1 }";  # i.e. drop trailing ";"
+		}
+		# otherwise protect expression from trailing ";"
+		return "do { $ass }"
 	}
 	
+	# Otherwise need to close over $type
+	$opts->{inline_environment}{'$type'} = \$type;
+	
+	# non-Type::Tiny but still supports inlining
 	if ( blessed($type)
-	and $type->can('assert_valid') )
+	and $type->can('can_be_inlined')
+	and $type->can_be_inlined )
 	{
-		$opts->{inline_environment}{'$type'} = \$type;
-		return sprintf('$type->assert_valid(%s)', $var);
+		my $inliner = $type->can('inline_check') || $type->can('_inline_check');
+		if ($inliner)
+		{
+			return sprintf('do { %s } ? %s : Carp::croak($type->get_message(%s))', $type->$inliner($var), $var, $var);
+		}
 	}
 	
 	if ( blessed($type)
 	and $type->can('check')
 	and $type->can('get_message') )
 	{
-		$opts->{inline_environment}{'$type'} = \$type;
-		return sprintf('$type->check(%s) or Carp::croak($type->get_message(%s))', $var, $var);
+		return sprintf('$type->check(%s) ? %s : Carp::croak($type->get_message(%s))', $var, $var, $var);
 	}
 	
-	$opts->{inline_environment}{'$type'} = \$type;
-	return sprintf('$type->(%s)', $var);
+	return sprintf('$type->(%s) ? %s : Carp::croak("Value %s failed type constraint check")', $var, $var, $var);
 }
 
 sub _inline_lexical_weaken : method
@@ -546,6 +613,8 @@ __END__
 
 =encoding utf-8
 
+=for stopwords benchmarking
+
 =head1 NAME
 
 Lexical::Accessor - true private attributes for Moose/Moo/Mouse
@@ -666,7 +735,7 @@ C<does> option.
 
 =item C<< coerce >>
 
-A coderef of L<Type::Coercion> object is accepted.
+A coderef or L<Type::Coercion> object is accepted.
 
 If the special value C<< '1' >> is provided, the type constraint object
 is consulted to find the coercion. (This doesn't work for coderef type
@@ -748,6 +817,26 @@ This function may also be called as a class method.
 
 =back
 
+=head2 Comparison (benchmarking, etc)
+
+Lexical::Accessor is almost three times faster than
+L<MooX::PrivateAttributes>, and almost twenty time faster than
+L<MooseX::Privacy>. I'd also argue that it's a more "correct"
+implementation of private accessors as (short of performing impressive
+L<PadWalker> manipulations), the accessors generated by this module
+are completely invisible to subclasses, method dispatch, etc.
+
+Compared to the usual Moose convention of using a leading underscore
+to indicate a private method (which is a very loose convention; it is
+quite common for subclasses to override such methods!),
+L<Lexical::Accessor> clearly offers much better method privacy. There
+should be little performance hit from using lexical accessors compared
+to normal L<Moose> accessors. (However they are nowhere near the speed
+of the XS-powered accessors that L<Moo> I<sometimes> uses and L<Mouse>
+I<usually> uses.)
+
+See also: C<< examples/benchmark.pl >> bundled with this release.
+
 =head1 BUGS
 
 Please report any bugs to
